<?php

/**
 * @file
 * Migrate pages given in xml format.
 */

/**
 * Implementation of WordPressMigration, for pages.
 */

class WordPressPostMigration extends XMLMigration {

  public $contenttype = 'post';
  public $dependencies = array('WordPressCategory', 'WordPressTag',
    'WordPressAuthor');

  public $description = 'Wordpress post exported as XML';

  public $xmlfolder = 'migrate/inbox/posts_xml';
  public $staticfolder = 'migrate/inbox/static';

  public $defaultfilterformat = 2;
  public $defaultuid = 1;
  public $defaultcommentsetting = 2;

  public $fields = array(
    'id' => 'Legacy ID',
    'legacy_url' => 'Legacy URL',
    'title' => 'Title',
    'deck' => 'Deck',
    'body' => 'Body',
    'coverdate' => 'Cover Date',
    'pubdate' => 'Publish Date',
    'expiredate' => 'Expire Date',
    'byline' => 'Byline',
    'authors' => 'Authors',
    'teaser' => 'Teaser',
    'promo_image' => 'Promo Image',
    'categories' => 'Categories',
    'primary_term' => 'Primary Term',
    'keywords' => 'Keywords',
    'status' => 'Publish Status',
    'urls' => 'Old Site Urls',
    'meta_description' => 'Description Meta Header',
    'meta_keywords' => 'Keywords Meta Header',
    'content_source' => 'Content Source (Brand)',
    'syndicate' => 'Syndication OK',
  );

  /**
   * Set it up : Initialisation.
   */
  public function __construct() {
    parent::__construct();

    $xml_path = file_directory_path() . '/' . $this->xmlfolder;

    // The source ID here is the one retrieved from the XML listing file, and
    // used to identify the specific item's file.
    $this->map = new MigrateSQLMap(
      $this->machineName,
      array(
        'sourceid' => array(
          'type' => 'int',
          'unsigned' => TRUE,
          'not null' => TRUE,
        ),
      ),
      MigrateDestinationNode::getKeySchema()
    );

    // Each ID retrieved from the list URL will be plugged into :id in the
    // item URL to fetch the specific objects.
    $item_url = $xml_path . '/:id.xml';
    // We use the MigrateSourceList class for any source where we have the list
    // of IDs to process separately from the data for each item. The listing
    // and item are represented by separate classes, so for example we could
    // replace the XML listing with a file directory listing, or the XML item
    // with a JSON item.
    $this->source = new MigrateSourceList(new WordpressMigrateListFiles($xml_path),
      new MigrateItemXML($item_url), $this->fields);
    $this->destination = new MigrateDestinationNode($this->contenttype);
    $this->setupmap();
  }

  /**
   * Custom function defining source and destination mapping.
   */
  public function setupmap() {

    $date_args = array(
      'timezone' => 'America/Chicago',
      'language' => LANGUAGE_NONE);

    $static_path = file_directory_path() . '/' . $this->staticfolder;
    $static_args = MigrateFileFieldHandler::arguments($static_path);

    $this->addFieldMapping('format')
         ->defaultValue($this->defaultfilterformat);
    $this->addFieldMapping('uid')
         ->defaultValue($this->defaultuid);
    $this->addFieldMapping('comment')
         ->defaultValue($this->defaultcommentsetting);

    $this->addFieldMapping('status', 'status');
    $this->addFieldMapping('created', 'coverdate');
    $this->addFieldMapping('title', 'title');
    $this->addFieldMapping('field_deck', 'deck');
    $this->addFieldMapping('body', 'body');
    $this->addFieldMapping('field_pubdate', 'coverdate')
      ->arguments($date_args);
    $this->addFieldMapping('field_byline', 'byline');
    $this->addFieldMapping('field_author', 'authors')
      ->sourceMigration('WordPressAuthor');
    $this->addFieldMapping('teaser', 'teaser');
    $this->addFieldMapping('field_image_promo', 'promo_image')
      ->arguments($static_args);
    $this->addFieldMapping('page_title', 'page_title');

    $this->addFieldMapping('Category', 'categories')
      ->sourceMigration('WordPressCategory')
      ->arguments(array('source_type' => 'tid'));
    $this->addFieldMapping('primaryterm', 'primary_term')
      ->sourceMigration('WordPressCategory');

    $this->addFieldMapping('Keywords', 'keywords')
      ->sourceMigration('WordPressTag');

    $this->addFieldMapping('path', 'legacy_url');
    $this->addFieldMapping('pathauto_perform_alias')
      ->defaultValue(0);

    $this->addFieldMapping('nodewords_description', 'meta_description');
    $this->addFieldMapping('nodewords_keywords', 'meta_keywords');

    $this->addFieldMapping('field_content_source', 'content_source');
    $this->addFieldMapping('field_syndicate', 'syndicate');
  }

  /**
   * Modify source row and get data from xml.
   */
  public function prepareRow($row) {
    $row->id = (string) $row->xml->doc_id['id'];
    $row->legacy_url = ltrim((string) $row->xml->primary_url['value'], '/');
    $row->title = (string) $row->xml->title;
    $row->deck = (string) $row->xml->deck;
    $row->body = (string) $row->xml->body;

    $row->coverdate = (string) $row->xml->cover_date['iso'];
    $row->pubdate = (string) $row->xml->published_date['iso'];
    $row->expiredate = (string) $row->xml->expire_date['iso'];

    if (!$row->pubdate) {
      $row->pubdate = $row->coverdate;
    }
    elseif (!$row->coverdate) {
      $row->coverdate = $row->pubdate;
    }

    if (!$row->pubdate || strtotime($row->pubdate) > time()) {
      // Not time to publish yet.
      $row->status = 0;
    }
    elseif ($row->expiredate && strtotime($row->expiredate) < time()) {
      // Already expired.
      $row->status = 0;
    }
    else {
      // Go ahead and publish.
      $row->status = 1;
    }

    $row->byline = (string) $row->xml->byline;
    $row->teaser = (string) $row->xml->promo_teaser;
    $row->page_title = (string) $row->xml->promo_title;
    $row->promo_image = (string) $row->xml->promo_image['href'];

    $row->authors = array();
    foreach ($row->xml->authors->author as $auth) {
      $row->authors[] = (string) $auth['id'];
    }

    $row->categories = array();
    foreach ($row->xml->categories->category as $cat) {
      $row->categories[] = (int) $cat['id'];
      if ($cat['class'] == 'primary') {
        $row->primary_term = (int) $cat['id'];
      }
    }

    $row->keywords = array();
    foreach ($row->xml->keywords->keyword as $key) {
      $row->keywords[] = (int) $key['id'];
    }

    $row->urls = array();
    foreach ($row->xml->all_urls->url as $url) {
      $row->urls[] = (string) $url['value'];
    }

    $row->meta_description = (string) $row->xml->description;
    $row->meta_keywords = (string) $row->xml->meta_keywords;

    $pubname = (string) $row->xml->publication['name'];
    if ($pubname) {
      $row->content_source = $this->getormakecontentsource($pubname);
    }

    if (strtolower((string) $row->xml->syndication['ok']) == 'true') {
      $row->syndicate = 1;
    }
    else {
      $row->syndicate = 0;
    }
  }

  /**
   * Modify node object and add author nid and uid.
   */
  public function prepare($node, $row) {
    if (is_array($node->field_author)) {
      $author_nid = $node->field_author[0]['nid'];
      $author_uid = db_result(db_query("select uid from {node} where nid = %d", $author_nid));
      if ($author_uid) {
        $node->uid = $author_uid;
      }
    }

    if (module_exists('scheduler')) {
      $now = time();
      if ($row->pubdate) {
        $pubdate_timestamp = strtotime($row->pubdate);
        if ($pubdate_timestamp > $now) {
          $node->publish_on = $pubdate_timestamp;
        }
      }
      if ($row->expiredate) {
        $expiredate_timestamp = strtotime($row->expiredate);
        if ($expiredate_timestamp > $now) {
          $node->unpublish_on = $expiredate_timestamp;
        }
      }
    }

    // On node_save()
    // so pre-emptively force pubdate to match created here.
    if (is_array($node->field_pubdate) && is_array($node->field_pubdate[0])) {
      $node->field_pubdate[0]['value'] = strftime('%Y-%m-%dT%H:%M:%S', $node->created);
    }
  }

  /**
   * Get or make content source with given title.
   */
  public function getormakecontentsource($title) {
    static $lookup = array();
    if (array_key_exists($title, $lookup)) {
      $nid = $lookup[$title];
    }
    else {
      $nid = db_result(db_query("select nid from {node} where type = 'content_source' and title = '%s'", $title));
      if (!$nid) {
        print "Creating new content_source node, ignore following warnings for missing fields\n";
        $node = new stdClass();
        $node->type = 'content_source';
        $node->title = $title;
        $node->uid = 1;
        node_save($node);
        $nid = $node->nid;
        print "Created new content_source node, ignore previous warnings for missing fields\n";
        print "$nid $title\n";
      }
      $lookup[$title] = $nid;
    }
    return $nid;
  }
}
